@@ -2,17 +2,18 @@ use std::io::{ErrorKind, Read, Write};
 use std::mem;
 use std::process::exit;
 
+use cranelift::codegen::settings::SetError;
 use cranelift::prelude::*;
 use cranelift_codegen::binemit::{NullStackMapSink, NullTrapSink};
 use cranelift_codegen::ir::FuncRef;
 use cranelift_codegen::settings::{self, Configurable};
 use cranelift_jit::{JITBuilder, JITModule};
-use cranelift_module::{default_libcall_names, FuncId, Linkage, Module};
+use cranelift_module::{default_libcall_names, FuncId, Linkage, Module, ModuleError};
 
 use crate::errors::CompilerError;
-use crate::ir::ops::{Op, OpType, LoopDecrement};
-use crate::OptimizeConfig;
+use crate::ir::ops::{LoopDecrement, Op, OpType};
 use crate::parser::Program;
+use crate::OptimizeConfig;
 
 struct Builder<'a> {
     pointer_type: Type,
@@ -35,24 +36,47 @@ impl<'a> Builder<'a> {
                 OpType::Inc(offset, value) => self.inc(*offset, *value),
                 OpType::Dec(offset, value) => self.dec(*offset, *value),
                 OpType::Set(offset, value) => self.set(*offset, *value),
-                OpType::Add(src_offset, dest_offset, multi) => self.add(*src_offset, *dest_offset, *multi),
-                OpType::NzAdd(src_offset, dest_offset, multi) => self.nz_add(*src_offset, *dest_offset, *multi),
-                OpType::CAdd(src_offset, dest_offset, value) => self.c_add(*src_offset, *dest_offset, *value),
-                OpType::NzCAdd(src_offset, dest_offset, value) => self.nz_c_add(*src_offset, *dest_offset, *value),
-                OpType::Sub(src_offset, dest_offset, multi) => self.sub(*src_offset, *dest_offset, *multi),
-                OpType::NzSub(src_offset, dest_offset, multi) => self.nz_sub(*src_offset, *dest_offset, *multi),
-                OpType::CSub(src_offset, dest_offset, value) => self.c_sub(*src_offset, *dest_offset, *value),
-                OpType::NzCSub(src_offset, dest_offset, value) => self.nz_c_sub(*src_offset, *dest_offset, *value),
-                OpType::Mul(src_offset, dest_offset, multi) => self.mul(*src_offset, *dest_offset, *multi),
-                OpType::NzMul(src_offset, dest_offset, multi) => self.nz_mul(*src_offset, *dest_offset, *multi),
+                OpType::Add(src_offset, dest_offset, multi) => {
+                    self.add(*src_offset, *dest_offset, *multi)
+                }
+                OpType::NzAdd(src_offset, dest_offset, multi) => {
+                    self.nz_add(*src_offset, *dest_offset, *multi)
+                }
+                OpType::CAdd(src_offset, dest_offset, value) => {
+                    self.c_add(*src_offset, *dest_offset, *value)
+                }
+                OpType::NzCAdd(src_offset, dest_offset, value) => {
+                    self.nz_c_add(*src_offset, *dest_offset, *value)
+                }
+                OpType::Sub(src_offset, dest_offset, multi) => {
+                    self.sub(*src_offset, *dest_offset, *multi)
+                }
+                OpType::NzSub(src_offset, dest_offset, multi) => {
+                    self.nz_sub(*src_offset, *dest_offset, *multi)
+                }
+                OpType::CSub(src_offset, dest_offset, value) => {
+                    self.c_sub(*src_offset, *dest_offset, *value)
+                }
+                OpType::NzCSub(src_offset, dest_offset, value) => {
+                    self.nz_c_sub(*src_offset, *dest_offset, *value)
+                }
+                OpType::Mul(src_offset, dest_offset, multi) => {
+                    self.mul(*src_offset, *dest_offset, *multi)
+                }
+                OpType::NzMul(src_offset, dest_offset, multi) => {
+                    self.nz_mul(*src_offset, *dest_offset, *multi)
+                }
                 OpType::Move(src_offset, dest_offset) => self._move(*src_offset, *dest_offset),
                 OpType::Copy(src_offset, dest_offset) => self.copy(*src_offset, *dest_offset),
                 OpType::DLoop(ops, _) => self.d_loop(ops),
                 OpType::LLoop(ops, _) => self.l_loop(ops),
                 OpType::ILoop(ops, step, decrement, _) => self.i_loop(ops, *step, *decrement),
-                OpType::CLoop(ops, iterations, decrement, _) => self.c_loop(ops, *iterations, *decrement),
+                OpType::CLoop(ops, iterations, decrement, _) => {
+                    self.c_loop(ops, *iterations, *decrement)
+                }
                 OpType::TNz(ops, _) => self.tnz(ops),
-                OpType::SearchZero(step) => self.search_zero(*step),
+                OpType::DTNz(ops, _, _) => self.d_tnz(ops),
+                OpType::SearchZero(step, _) => self.search_zero(*step),
                 OpType::PutChar(offset) => self.put_char(*offset),
                 OpType::PutString(array) => self.put_string(array),
                 OpType::GetChar(offset) => self.get_char(*offset),
@@ -65,11 +89,15 @@ impl<'a> Builder<'a> {
     }
 
     fn load(&mut self, offset: isize) -> Value {
-        self.bcx.ins().load(types::I8, MemFlags::new(), self.heap_ptr, offset as i32)
+        self.bcx
+            .ins()
+            .load(types::I8, MemFlags::new(), self.heap_ptr, offset as i32)
     }
 
     fn store(&mut self, offset: isize, value: Value) {
-        self.bcx.ins().store(MemFlags::new(), value, self.heap_ptr, offset as i32);
+        self.bcx
+            .ins()
+            .store(MemFlags::new(), value, self.heap_ptr, offset as i32);
     }
 
     fn inc_ptr(&mut self, value: usize) {
@@ -237,6 +265,38 @@ impl<'a> Builder<'a> {
         self.set(0, 0);
     }
 
+    fn d_tnz(&mut self, ops: &[Op]) {
+        let head = self.bcx.create_block();
+        self.bcx.append_block_param(head, self.pointer_type);
+
+        let body = self.bcx.create_block();
+        self.bcx.append_block_param(body, self.pointer_type);
+
+        let next = self.bcx.create_block();
+        self.bcx.append_block_param(next, self.pointer_type);
+
+        self.bcx.ins().fallthrough(head, &[self.heap_ptr]);
+
+        // Head with condition
+        self.bcx.switch_to_block(head);
+        self.heap_ptr = self.bcx.block_params(head)[0];
+
+        let value = self.load(0);
+        self.bcx.ins().brz(value, next, &[self.heap_ptr]);
+        self.bcx.ins().fallthrough(body, &[self.heap_ptr]);
+
+        // Loop Body
+        self.bcx.switch_to_block(body);
+        self.append_ops(ops);
+        self.bcx.ins().fallthrough(next, &[self.heap_ptr]);
+
+        // Start next block after loop
+        self.bcx.switch_to_block(next);
+        self.heap_ptr = self.bcx.block_params(next)[0];
+        // This is not be needed but it seems to fix a possible bug in cranelift
+        self.set(0, 0);
+    }
+
     fn l_loop(&mut self, ops: &[Op]) {
         let head = self.bcx.create_block();
         self.bcx.append_block_param(head, self.pointer_type);
@@ -286,7 +346,6 @@ impl<'a> Builder<'a> {
         let next = self.bcx.create_block();
         self.bcx.append_block_param(next, self.pointer_type);
 
-
         let value = self.load(0);
 
         self.bcx.ins().fallthrough(head, &[self.heap_ptr, value]);
@@ -350,7 +409,9 @@ impl<'a> Builder<'a> {
             self.store(0, iterations);
         }
 
-        self.bcx.ins().fallthrough(head, &[self.heap_ptr, iterations]);
+        self.bcx
+            .ins()
+            .fallthrough(head, &[self.heap_ptr, iterations]);
 
         // Head with condition
         self.bcx.switch_to_block(head);
@@ -472,10 +533,7 @@ struct Environment<'a, 'b> {
 
 impl<'a, 'b> Environment<'a, 'b> {
     pub fn new(input: &'a mut dyn Read, output: &'b mut dyn Write) -> Environment<'a, 'b> {
-        Environment {
-            input,
-            output,
-        }
+        Environment { input, output }
     }
 }
 
@@ -506,6 +564,22 @@ fn put_char(env: *mut Environment, value: u8) {
     }
 }
 
+impl From<SetError> for CompilerError {
+    fn from(e: SetError) -> Self {
+        CompilerError::InternalCompilerError {
+            message: e.to_string(),
+        }
+    }
+}
+
+impl From<ModuleError> for CompilerError {
+    fn from(e: ModuleError) -> Self {
+        CompilerError::InternalCompilerError {
+            message: e.to_string(),
+        }
+    }
+}
+
 /// A compiled program that can be executed
 pub struct CompiledJitModule {
     module: Option<JITModule>,
@@ -515,15 +589,18 @@ pub struct CompiledJitModule {
 
 impl CompiledJitModule {
     /// Compile program
-    pub fn new(program: &Program, opt_mode: &OptimizeConfig) -> Result<CompiledJitModule, CompilerError> {
+    pub fn new(
+        program: &Program,
+        opt_mode: &OptimizeConfig,
+    ) -> Result<CompiledJitModule, CompilerError> {
         let mut flag_builder = settings::builder();
 
         if let Some(jit_level) = &opt_mode.jit_level {
-            flag_builder.set("opt_level", jit_level).unwrap();
+            flag_builder.set("opt_level", jit_level)?;
         } else if opt_mode.optimize() {
-            flag_builder.set("opt_level", "speed").unwrap();
+            flag_builder.set("opt_level", "speed")?;
         } else {
-            flag_builder.set("opt_level", "none").unwrap();
+            flag_builder.set("opt_level", "none")?;
         }
 
         let isa_builder = cranelift_native::builder().unwrap_or_else(|msg| {
@@ -543,13 +620,13 @@ impl CompiledJitModule {
         get_char_sig.params.push(AbiParam::new(pointer_type));
         get_char_sig.returns.push(AbiParam::new(types::I8));
 
-        let get_char_func = module.declare_function("get_char", Linkage::Import, &get_char_sig).unwrap();
+        let get_char_func = module.declare_function("get_char", Linkage::Import, &get_char_sig)?;
 
         let mut put_char_sig = module.make_signature();
         put_char_sig.params.push(AbiParam::new(pointer_type));
         put_char_sig.params.push(AbiParam::new(types::I8));
 
-        let put_char_func = module.declare_function("put_char", Linkage::Import, &put_char_sig).unwrap();
+        let put_char_func = module.declare_function("put_char", Linkage::Import, &put_char_sig)?;
 
         let mut ctx = module.make_context();
         let mut func_ctx = FunctionBuilderContext::new();
@@ -557,14 +634,11 @@ impl CompiledJitModule {
         let mut trap_sink = NullTrapSink {};
         let mut stack_map_sink = NullStackMapSink {};
 
-
         let mut sig = module.make_signature();
         sig.params.push(AbiParam::new(pointer_type));
         sig.params.push(AbiParam::new(pointer_type));
 
-        let func = module
-            .declare_function("main", Linkage::Local, &sig)
-            .unwrap();
+        let func = module.declare_function("main", Linkage::Local, &sig)?;
 
         ctx.func.signature = sig;
         ctx.func.name = ExternalName::user(0, func.as_u32());
@@ -605,7 +679,7 @@ impl CompiledJitModule {
                 if let Err((_inst, msg)) = bcx.func.is_block_basic(block) {
                     println!("{}", bcx.func);
                     return Err(CompilerError::InternalCompilerError {
-                        message: format!("Bad block: {}, {}", msg, block)
+                        message: format!("Bad block: {}, {}", msg, block),
                     });
                 }
             }
@@ -615,9 +689,7 @@ impl CompiledJitModule {
             format!("{:?}", bcx.func)
         };
 
-        module
-            .define_function(func, &mut ctx, &mut trap_sink, &mut stack_map_sink)
-            .unwrap();
+        module.define_function(func, &mut ctx, &mut trap_sink, &mut stack_map_sink)?;
         module.clear_context(&mut ctx);
 
         module.finalize_definitions();
@@ -670,15 +742,17 @@ impl Drop for CompiledJitModule {
 mod tests {
     use std::io::{Cursor, Read, Write};
 
-    use crate::{OptimizeConfig, parse, optimize_with_config};
-    use crate::ir::ops::{Op, LoopDecrement};
+    use crate::ir::ops::{LoopDecrement, Op};
+    use crate::ir::opt_info::BlockInfo;
     use crate::parser::Program;
+    use crate::{optimize_with_config, parse, OptimizeConfig};
 
     use super::CompiledJitModule;
-    use crate::ir::opt_info::BlockInfo;
 
     fn run<R: Read, W: Write>(program: &Program, input: R, output: W) -> Vec<u8> {
-        CompiledJitModule::new(program, &OptimizeConfig::o2()).unwrap().execute(input, output)
+        CompiledJitModule::new(program, &OptimizeConfig::o2())
+            .unwrap()
+            .execute(input, output)
     }
 
     #[test]
@@ -686,7 +760,6 @@ mod tests {
         let mut program = parse("++>>++++-<-").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -702,7 +775,6 @@ mod tests {
         let mut program = parse(",+.").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"a";
         let mut output = Vec::new();
 
@@ -713,14 +785,15 @@ mod tests {
 
     #[test]
     fn test_hello_world() {
-        let mut program = parse("
+        let mut program = parse(
+            "
                 >+++++++++[<++++++++>-]<.>+++++++[<++++>-]<+.+++++++..+++.[-]
                 >++++++++[<++++>-] <.>+++++++++++[<++++++++>-]<-.--------.+++
-                .------.--------.[-]>++++++++[<++++>- ]<+.[-]++++++++++."
-        ).unwrap();
+                .------.--------.[-]>++++++++[<++++>- ]<+.[-]++++++++++.",
+        )
+        .unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -731,11 +804,9 @@ mod tests {
 
     #[test]
     fn test_dloop() {
-        let mut program = parse(">,<++[->+<]>."
-        ).unwrap();
+        let mut program = parse(">,<++[->+<]>.").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"1";
         let mut output = Vec::new();
 
@@ -749,7 +820,6 @@ mod tests {
         let mut program = parse(include_str!("../../../test_programs/hello_world.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -763,7 +833,6 @@ mod tests {
         let mut program = parse("++++[->++<]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -778,7 +847,6 @@ mod tests {
         let mut program = parse("++++[->++<]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -793,7 +861,6 @@ mod tests {
         let mut program = parse("").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -807,7 +874,6 @@ mod tests {
         let mut program = parse("[>+++++<-]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -821,7 +887,6 @@ mod tests {
         let mut program = parse(">>++<<++[>+++++<-]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -836,7 +901,6 @@ mod tests {
         let mut program = parse(">>++<<++++++++++++[>+<---]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -851,7 +915,6 @@ mod tests {
         let mut program = parse("+++++[>+++++[>+++++<-]<-]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -867,7 +930,6 @@ mod tests {
         let mut program = parse("++++++++[---->+<++]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -882,7 +944,6 @@ mod tests {
         let mut program = parse("+>>++<<[->[-]++++++<]").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -898,7 +959,6 @@ mod tests {
         let mut program = parse("++++[-]++").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -912,7 +972,6 @@ mod tests {
         let mut program = parse("+++++++++++++[>+>++>>+++<<<<-]->>>>>>+").unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -932,7 +991,6 @@ mod tests {
         let mut program = parse(include_str!("../../../test_programs/fizz.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -946,7 +1004,6 @@ mod tests {
         let mut program = parse(include_str!("../../../test_programs/fizzbuzz.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -960,13 +1017,15 @@ mod tests {
         let mut program = parse(include_str!("../../../test_programs/bottles.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
         let _heap = run(&program, Cursor::new(input), &mut output);
 
-        assert_eq!(output, include_bytes!("../../../test_programs/bottles.bf.out"));
+        assert_eq!(
+            output,
+            include_bytes!("../../../test_programs/bottles.bf.out")
+        );
     }
 
     #[test]
@@ -974,13 +1033,15 @@ mod tests {
         let mut program = parse(include_str!("../../../test_programs/factor.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = include_bytes!("../../../test_programs/factor.bf.in");
         let mut output = Vec::new();
 
         let _heap = run(&program, Cursor::new(input), &mut output);
 
-        assert_eq!(output, include_bytes!("../../../test_programs/factor.bf.out"));
+        assert_eq!(
+            output,
+            include_bytes!("../../../test_programs/factor.bf.out")
+        );
     }
 
     #[test]
@@ -988,7 +1049,6 @@ mod tests {
         let mut program = parse(include_str!("../../../test_programs/life.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = include_bytes!("../../../test_programs/life.bf.in");
         let mut output = Vec::new();
 
@@ -1006,7 +1066,7 @@ mod tests {
                 Op::set(0..1, 3),
                 Op::dec_ptr(1..2, 1),
                 Op::set(0..1, 2),
-            ]
+            ],
         };
 
         let input = b"";
@@ -1026,8 +1086,8 @@ mod tests {
             ops: vec![
                 Op::set(0..1, 5),
                 Op::nz_add(1..2, 1, 1),
-                Op::nz_add(2..3, 3, 2)
-            ]
+                Op::nz_add(2..3, 3, 2),
+            ],
         };
 
         let input = b"";
@@ -1044,11 +1104,7 @@ mod tests {
     #[test]
     fn test_add() {
         let program = Program {
-            ops: vec![
-                Op::set(0..1, 5),
-                Op::add(1..2, 1, 2),
-                Op::add(2..3, 3, 1)
-            ]
+            ops: vec![Op::set(0..1, 5), Op::add(1..2, 1, 2), Op::add(2..3, 3, 1)],
         };
 
         let input = b"";
@@ -1069,8 +1125,8 @@ mod tests {
                 Op::set(0..1, 5),
                 Op::nz_add(1..2, 1, 1),
                 Op::nz_add(2..3, 3, 2),
-                Op::add(2..3, 4, 3)
-            ]
+                Op::add(2..3, 4, 3),
+            ],
         };
 
         let input = b"";
@@ -1091,8 +1147,8 @@ mod tests {
             ops: vec![
                 Op::set(0..1, 2),
                 Op::nz_c_add(1..2, 1, 4),
-                Op::nz_c_add(2..3, 3, 8)
-            ]
+                Op::nz_c_add(2..3, 3, 8),
+            ],
         };
 
         let input = b"";
@@ -1112,8 +1168,8 @@ mod tests {
             ops: vec![
                 Op::set(0..1, 2),
                 Op::c_add(1..2, 1, 4),
-                Op::c_add(2..3, 3, 8)
-            ]
+                Op::c_add(2..3, 3, 8),
+            ],
         };
 
         let input = b"";
@@ -1130,11 +1186,7 @@ mod tests {
     #[test]
     fn test_sub() {
         let program = Program {
-            ops: vec![
-                Op::set(0..1, 5),
-                Op::sub(1..2, 1, 2),
-                Op::sub(2..3, 3, 1)
-            ]
+            ops: vec![Op::set(0..1, 5), Op::sub(1..2, 1, 2), Op::sub(2..3, 3, 1)],
         };
 
         let input = b"";
@@ -1155,8 +1207,8 @@ mod tests {
                 Op::set(0..1, 5),
                 Op::nz_sub(1..2, 1, 1),
                 Op::nz_sub(2..3, 3, 2),
-                Op::sub(2..3, 4, 3)
-            ]
+                Op::sub(2..3, 4, 3),
+            ],
         };
 
         let input = b"";
@@ -1177,8 +1229,8 @@ mod tests {
             ops: vec![
                 Op::set(0..1, 2),
                 Op::nz_c_sub(1..2, 1, 4),
-                Op::nz_c_sub(2..3, 3, 8)
-            ]
+                Op::nz_c_sub(2..3, 3, 8),
+            ],
         };
 
         let input = b"";
@@ -1198,8 +1250,8 @@ mod tests {
             ops: vec![
                 Op::set(0..1, 2),
                 Op::c_sub(1..2, 1, 4),
-                Op::c_sub(2..3, 3, 8)
-            ]
+                Op::c_sub(2..3, 3, 8),
+            ],
         };
 
         let input = b"";
@@ -1216,11 +1268,7 @@ mod tests {
     #[test]
     fn test_mul() {
         let program = Program {
-            ops: vec![
-                Op::set(0..1, 5),
-                Op::mul(1..2, 1, 2),
-                Op::mul(2..3, 3, 1)
-            ]
+            ops: vec![Op::set(0..1, 5), Op::mul(1..2, 1, 2), Op::mul(2..3, 3, 1)],
         };
 
         let input = b"";
@@ -1238,8 +1286,8 @@ mod tests {
                 Op::set(0..1, 5),
                 Op::nz_mul(1..2, 1, 2),
                 Op::nz_mul(2..3, 3, 3),
-                Op::mul(2..3, 4, 4)
-            ]
+                Op::mul(2..3, 4, 4),
+            ],
         };
 
         let input = b"";
@@ -1253,10 +1301,7 @@ mod tests {
     #[test]
     fn test_move() {
         let program = Program {
-            ops: vec![
-                Op::set(0..1, 2),
-                Op::_move(1..2, 0, 1),
-            ]
+            ops: vec![Op::set(0..1, 2), Op::_move(1..2, 0, 1)],
         };
 
         let input = b"";
@@ -1272,10 +1317,7 @@ mod tests {
     #[test]
     fn test_copy() {
         let program = Program {
-            ops: vec![
-                Op::set(0..1, 2),
-                Op::copy(1..2, 0, 1),
-            ]
+            ops: vec![Op::set(0..1, 2), Op::copy(1..2, 0, 1)],
         };
 
         let input = b"";
@@ -1291,9 +1333,7 @@ mod tests {
     #[test]
     fn test_get_char() {
         let program = Program {
-            ops: vec![
-                Op::get_char(0..1),
-            ]
+            ops: vec![Op::get_char(0..1)],
         };
 
         let input = b"\x02";
@@ -1308,10 +1348,7 @@ mod tests {
     #[test]
     fn test_put_char() {
         let program = Program {
-            ops: vec![
-                Op::set(0..1, 2),
-                Op::put_char(1..2),
-            ]
+            ops: vec![Op::set(0..1, 2), Op::put_char(1..2)],
         };
 
         let input = b"";
@@ -1328,14 +1365,18 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 5),
-                Op::d_loop(1..4, vec![
-                    Op::dec(1..2, 1),
-                    Op::inc_ptr(1..2, 1),
-                    Op::inc(1..2, 1),
-                    Op::inc(1..2, 1),
-                    Op::dec_ptr(1..2, 1),
-                ], BlockInfo::new_empty()),
-            ]
+                Op::d_loop(
+                    1..4,
+                    vec![
+                        Op::dec(1..2, 1),
+                        Op::inc_ptr(1..2, 1),
+                        Op::inc(1..2, 1),
+                        Op::inc(1..2, 1),
+                        Op::dec_ptr(1..2, 1),
+                    ],
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1352,15 +1393,19 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 5),
-                Op::l_loop(1..4, vec![
-                    Op::inc_ptr(1..2, 1),
-                    Op::dec_ptr(1..2, 1),
-                    Op::dec(1..2, 1),
-                    Op::inc_ptr(1..2, 2),
-                    Op::inc(1..2, 1),
-                    Op::inc(1..2, 1),
-                ], BlockInfo::new_empty()),
-            ]
+                Op::l_loop(
+                    1..4,
+                    vec![
+                        Op::inc_ptr(1..2, 1),
+                        Op::dec_ptr(1..2, 1),
+                        Op::dec(1..2, 1),
+                        Op::inc_ptr(1..2, 2),
+                        Op::inc(1..2, 1),
+                        Op::inc(1..2, 1),
+                    ],
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1378,12 +1423,13 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 5),
-                Op::i_loop(1..4, vec![
-                    Op::inc_ptr(1..2, 2),
-                    Op::inc(1..2, 1),
-                    Op::inc(1..2, 1),
-                ], 1, BlockInfo::new_empty()),
-            ]
+                Op::i_loop(
+                    1..4,
+                    vec![Op::inc_ptr(1..2, 2), Op::inc(1..2, 1), Op::inc(1..2, 1)],
+                    1,
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1401,12 +1447,13 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 10),
-                Op::i_loop(1..4, vec![
-                    Op::inc_ptr(1..2, 2),
-                    Op::inc(1..2, 1),
-                    Op::inc(1..2, 1),
-                ], 2, BlockInfo::new_empty()),
-            ]
+                Op::i_loop(
+                    1..4,
+                    vec![Op::inc_ptr(1..2, 2), Op::inc(1..2, 1), Op::inc(1..2, 1)],
+                    2,
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1424,13 +1471,19 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 5),
-                Op::i_loop_with_decrement(1..4, vec![
-                    Op::inc_ptr(1..2, 1),
-                    Op::inc(1..2, 1),
-                    Op::dec_ptr(1..2, 1),
-                    Op::put_char(1..2),
-                ], 1, LoopDecrement::Pre, BlockInfo::new_empty()),
-            ]
+                Op::i_loop_with_decrement(
+                    1..4,
+                    vec![
+                        Op::inc_ptr(1..2, 1),
+                        Op::inc(1..2, 1),
+                        Op::dec_ptr(1..2, 1),
+                        Op::put_char(1..2),
+                    ],
+                    1,
+                    LoopDecrement::Pre,
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1448,10 +1501,14 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 5),
-                Op::i_loop_with_decrement(1..4, vec![
-                    Op::put_char(1..2),
-                ], 1, LoopDecrement::Post, BlockInfo::new_empty()),
-            ]
+                Op::i_loop_with_decrement(
+                    1..4,
+                    vec![Op::put_char(1..2)],
+                    1,
+                    LoopDecrement::Post,
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1468,11 +1525,16 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 10),
-                Op::c_loop(1..4, vec![
-                    Op::inc_with_offset(1..2, 2, 1),
-                    Op::inc_with_offset(1..2, 2, 1),
-                ], 5, BlockInfo::new_empty()),
-            ]
+                Op::c_loop(
+                    1..4,
+                    vec![
+                        Op::inc_with_offset(1..2, 2, 1),
+                        Op::inc_with_offset(1..2, 2, 1),
+                    ],
+                    5,
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1488,14 +1550,18 @@ mod tests {
     #[test]
     fn test_c_loop_pre() {
         let program = Program {
-            ops: vec![
-                Op::c_loop_with_decrement(1..4, vec![
+            ops: vec![Op::c_loop_with_decrement(
+                1..4,
+                vec![
                     Op::inc_ptr(1..2, 1),
                     Op::inc(1..2, 1),
                     Op::dec_ptr(1..2, 1),
                     Op::put_char(1..2),
-                ], 5, LoopDecrement::Pre, BlockInfo::new_empty()),
-            ]
+                ],
+                5,
+                LoopDecrement::Pre,
+                BlockInfo::new_empty(),
+            )],
         };
 
         let input = b"";
@@ -1511,11 +1577,13 @@ mod tests {
     #[test]
     fn test_c_loop_post() {
         let program = Program {
-            ops: vec![
-                Op::c_loop_with_decrement(1..4, vec![
-                    Op::put_char(1..2),
-                ], 5, LoopDecrement::Post, BlockInfo::new_empty()),
-            ]
+            ops: vec![Op::c_loop_with_decrement(
+                1..4,
+                vec![Op::put_char(1..2)],
+                5,
+                LoopDecrement::Post,
+                BlockInfo::new_empty(),
+            )],
         };
 
         let input = b"";
@@ -1532,10 +1600,12 @@ mod tests {
         let program = Program {
             ops: vec![
                 Op::set(0..1, 1),
-                Op::t_nz(1..4, vec![
-                    Op::set_with_offset(1..2, 2, 10),
-                ], BlockInfo::new_empty()),
-            ]
+                Op::t_nz(
+                    1..4,
+                    vec![Op::set_with_offset(1..2, 2, 10)],
+                    BlockInfo::new_empty(),
+                ),
+            ],
         };
 
         let input = b"";
@@ -1551,11 +1621,11 @@ mod tests {
     #[test]
     fn test_tnz_false() {
         let program = Program {
-            ops: vec![
-                Op::t_nz(1..4, vec![
-                    Op::set_with_offset(1..2, 2, 10),
-                ], BlockInfo::new_empty()),
-            ]
+            ops: vec![Op::t_nz(
+                1..4,
+                vec![Op::set_with_offset(1..2, 2, 10)],
+                BlockInfo::new_empty(),
+            )],
         };
 
         let input = b"";
@@ -1580,7 +1650,7 @@ mod tests {
                 Op::set(1..2, 3),
                 Op::search_zero(0..1, -1),
                 Op::set(1..2, 4),
-            ]
+            ],
         };
 
         let input = b"";
@@ -1604,13 +1674,11 @@ mod tests {
         assert_eq!(output, b"0123456789aZ");
     }
 
-
     #[test]
     fn test_cell_size() {
         let mut program = parse(include_str!("../../../test_programs/cell_size.bf")).unwrap();
         optimize_with_config(&mut program, &OptimizeConfig::o3());
 
-
         let input = b"";
         let mut output = Vec::new();
 
@@ -1639,4 +1707,4 @@ mod tests {
         assert_eq!(o, o2);
     }
      */
-}
\ No newline at end of file
+}
